﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Linq.Expressions;
using System.Web.Mvc.Html;
using System.Text;
using System.IO;

using ComLib.Extensions;
using ComLib.Web.Lib.Core;


namespace ComLib.Web.Lib.Extensions
{
    /// <summary>
    /// Class to provided localized resource strings for labels.
    /// This is currently in place but using default data for now.
    /// </summary>
    public static class HtmlHelperExtensionsForResources
    {
        /// <summary>
        /// Get the localized resource string for the supplied expression.
        /// e.g. "Post", "Create" => Create a new BlogPost
        /// </summary>
        /// <param name="htmlhelper">Class used for extensions methods</param>
        /// <param name="expression">The expression associated w/ the resource</param>
        /// <returns></returns>
        public static MvcHtmlString ResourceFor<TModel>(this HtmlHelper<TModel> htmlhelper, Expression<Func<TModel, object>> expression)
        {
            return ResourceFor<TModel>(htmlhelper, expression, null, null);
        }


        /// <summary>
        /// Get the localized resource string for the supplied expression.
        /// e.g. "Post", "Create" => Create a new BlogPost
        /// </summary>
        /// <param name="htmlhelper">Class used for extensions methods</param>
        /// <param name="expression">The expression associated w/ the resource</param>
        /// <param name="defaultValue">The default value to use if the resource key is not there.</param>
        /// <returns></returns>
        public static MvcHtmlString ResourceFor<TModel>(this HtmlHelper<TModel> htmlhelper, Expression<Func<TModel, object>> expression, string defaultValue)
        {
            return ResourceFor<TModel>(htmlhelper, expression, defaultValue, null);
        }


        /// <summary>
        /// Get the localized resource string for the supplied expression.
        /// e.g. "Post", "Create" => Create a new BlogPost
        /// </summary>
        /// <param name="htmlhelper">Class used for extensions methods</param>
        /// <param name="expression">The expression associated w/ the resource</param>
        /// <param name="defaultValue">The default value to use if the resource key is not there.</param>
        /// <param name="args">The args to use on the resource string.</param>
        /// <returns></returns>
        public static MvcHtmlString ResourceFor<TModel>(this HtmlHelper<TModel> htmlhelper, Expression<Func<TModel, object>> expression, string defaultValue, params object[] args)
        {
            string section = typeof(TModel).Name;
            string key = ComLib.ExpressionHelper.GetPropertyName<TModel>(expression);
            MvcHtmlString text = !string.IsNullOrEmpty(defaultValue) ? htmlhelper.Label(defaultValue) : htmlhelper.Label(key);
            return text;
        }


        /// <summary>
        /// Get the localized resource string for the supplied expression.
        /// e.g. "Post", "Create" => Create a new BlogPost
        /// </summary>
        /// <param name="htmlhelper">Class used for extensions methods</param>
        /// <param name="key">The key associated w/ the resource</param>
        /// <returns></returns>
        public static MvcHtmlString ResourceFor<TModel>(this HtmlHelper<TModel> htmlhelper, string key)
        {
            return ResourceFor<TModel>(htmlhelper, key, null, null);
        }


        /// <summary>
        /// Get the localized resource string for the supplied expression.
        /// e.g. "Post", "Create" => Create a new BlogPost
        /// </summary>
        /// <param name="htmlhelper">Class used for extensions methods</param>
        /// <param name="key">The key associated w/ the resource</param>
        /// <param name="defaultvalue">The default value to use if the resource string is not available.</param>
        /// <returns></returns>
        public static MvcHtmlString ResourceFor<TModel>(this HtmlHelper<TModel> htmlhelper, string key, string defaultvalue)
        {
            return ResourceFor<TModel>(htmlhelper, key, defaultvalue, null);
        }


        /// <summary>
        /// Get the localized resource string for the supplied expression.
        /// e.g. "Post", "Create" => Create a new BlogPost
        /// </summary>
        /// <param name="htmlhelper">Class used for extensions methods</param>
        /// <param name="key">The key associated w/ the resource</param>
        /// <param name="defaultvalue">The default value to use if the resource string is not available.</param>
        /// <param name="args">The args to use on the resource string.</param>
        /// <returns></returns>
        public static MvcHtmlString ResourceFor<TModel>(this HtmlHelper<TModel> htmlhelper, string key, string defaultvalue, params object[] args)
        {
            string section = typeof(TModel).Name;
            MvcHtmlString text = htmlhelper.Label(key);
            return text;
        }        


        /// <summary>
        /// Get the localized resource string for the supplied section/key combination
        /// e.g. "Post", "Create" => Create a new BlogPost
        /// </summary>
        /// <param name="htmlhelper">Class used for extensions methods</param>
        /// <param name="section">The section associated w/ the resource</param>
        /// <param name="key">The key associated w/ the resource.</param>
        /// <returns></returns>
        public static MvcHtmlString Resource(this HtmlHelper htmlhelper, string section, string key)
        {
            return Resource(htmlhelper, section, key, null, null);
        }


        /// <summary>
        /// Get the localized resource string for the supplied section/key combination
        /// e.g. "Post", "Create" => Create a new BlogPost
        /// </summary>
        /// <param name="htmlhelper">Class used for extensions methods</param>
        /// <param name="section">The section associated w/ the resource</param>
        /// <param name="key">The key associated w/ the resource.</param>
        /// <param name="defaultvalue">The default value to use if the section/key is not there.</param>
        /// <returns></returns>
        public static MvcHtmlString Resource(this HtmlHelper htmlhelper, string section, string key, string defaultvalue)
        {
            return Resource(htmlhelper, section, key, defaultvalue, null);
        }


        /// <summary>
        /// Get the localized resource string for the supplied section/key combination
        /// e.g. "Post", "Create" => Create a new BlogPost
        /// </summary>
        /// <param name="htmlhelper">Class used for extensions methods</param>
        /// <param name="section">The section associated w/ the resource</param>
        /// <param name="key">The key associated w/ the resource.</param>
        /// <param name="defaultvalue">The default value to use if the section/key is not there.</param>
        /// <param name="args">The args to supply to the string.format() method to use the resource string.</param>
        /// <returns></returns>
        public static MvcHtmlString Resource(this HtmlHelper htmlhelper, string section, string key, string defaultvalue, params object[] args)
        {
            if (!string.IsNullOrEmpty(defaultvalue))
                return htmlhelper.Label(defaultvalue);

            return htmlhelper.Label(key);
        }
    }
}
